@@ -17,11 +17,13 @@ limitations under the License.
 package clusterapi
 
 import (
+	"reflect"
 	"strings"
 	"testing"
 
 	"k8s.io/apimachinery/pkg/api/resource"
-	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/autoscaler/cluster-autoscaler/utils/units"
 )
 
@@ -112,15 +114,41 @@ func TestUtilParseScalingBounds(t *testing.T) {
 		},
 		min: 0,
 		max: 1,
+	}, {
+		description: "deprecated min/max annotations still work, result is min 0, max 1",
+		annotations: map[string]string{
+			deprecatedNodeGroupMinSizeAnnotationKey: "0",
+			deprecatedNodeGroupMaxSizeAnnotationKey: "1",
+		},
+		min: 0,
+		max: 1,
+	}, {
+		description: "deprecated min/max annotations do not take precedence over non-deprecated annotations, result is min 1, max 2",
+		annotations: map[string]string{
+			deprecatedNodeGroupMinSizeAnnotationKey: "0",
+			deprecatedNodeGroupMaxSizeAnnotationKey: "1",
+			nodeGroupMinSizeAnnotationKey:           "1",
+			nodeGroupMaxSizeAnnotationKey:           "2",
+		},
+		min: 1,
+		max: 2,
 	}} {
 		t.Run(tc.description, func(t *testing.T) {
-			machineSet := MachineSet{
-				ObjectMeta: v1.ObjectMeta{
-					Annotations: tc.annotations,
+			machineSet := unstructured.Unstructured{
+				Object: map[string]interface{}{
+					"kind":       machineSetKind,
+					"apiVersion": "cluster.x-k8s.io/v1alpha3",
+					"metadata": map[string]interface{}{
+						"name":      "test",
+						"namespace": "default",
+					},
+					"spec":   map[string]interface{}{},
+					"status": map[string]interface{}{},
 				},
 			}
+			machineSet.SetAnnotations(tc.annotations)
 
-			min, max, err := parseScalingBounds(machineSet.Annotations)
+			min, max, err := parseScalingBounds(machineSet.GetAnnotations())
 			if tc.error != nil && err == nil {
 				t.Fatalf("test #%d: expected an error", i)
 			}
@@ -143,228 +171,244 @@ func TestUtilParseScalingBounds(t *testing.T) {
 	}
 }
 
-func TestUtilMachineSetIsOwnedByMachineDeployment(t *testing.T) {
+func TestUtilGetOwnerByKindMachineSet(t *testing.T) {
 	for _, tc := range []struct {
-		description       string
-		machineSet        MachineSet
-		machineDeployment MachineDeployment
-		owned             bool
+		description         string
+		machineSet          *unstructured.Unstructured
+		machineSetOwnerRefs []metav1.OwnerReference
+		expectedOwnerRef    *metav1.OwnerReference
 	}{{
-		description:       "not owned as no owner references",
-		machineSet:        MachineSet{},
-		machineDeployment: MachineDeployment{},
-		owned:             false,
+		description:         "not owned as no owner references",
+		machineSet:          &unstructured.Unstructured{},
+		machineSetOwnerRefs: []metav1.OwnerReference{},
+		expectedOwnerRef:    nil,
 	}, {
 		description: "not owned as not the same Kind",
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "Other",
-				}},
+		machineSet: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
 			},
 		},
-		machineDeployment: MachineDeployment{},
-		owned:             false,
+		machineSetOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: "Other",
+			},
+		},
+		expectedOwnerRef: nil,
 	}, {
 		description: "not owned because no OwnerReference.Name",
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineSet",
-					UID:  uuid1,
-				}},
+		machineSet: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
 			},
 		},
-		machineDeployment: MachineDeployment{
-			ObjectMeta: v1.ObjectMeta{
-				UID: uuid1,
+		machineSetOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: machineDeploymentKind,
+				UID:  uuid1,
 			},
 		},
-		owned: false,
-	}, {
-		description: "not owned as UID values don't match",
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineSet",
-					Name: "foo",
-					UID:  uuid2,
-				}},
-			},
-		},
-		machineDeployment: MachineDeployment{
-			TypeMeta: v1.TypeMeta{
-				Kind: "MachineDeployment",
-			},
-			ObjectMeta: v1.ObjectMeta{
-				UID: uuid1,
-			},
-		},
-		owned: false,
+		expectedOwnerRef: nil,
 	}, {
 		description: "owned as UID values match and same Kind and Name not empty",
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineDeployment",
-					Name: "foo",
-					UID:  uuid1,
-				}},
+		machineSet: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
 			},
 		},
-		machineDeployment: MachineDeployment{
-			TypeMeta: v1.TypeMeta{
-				Kind: "MachineDeployment",
-			},
-			ObjectMeta: v1.ObjectMeta{
+		machineSetOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: machineDeploymentKind,
 				Name: "foo",
 				UID:  uuid1,
 			},
 		},
-		owned: true,
+		expectedOwnerRef: &metav1.OwnerReference{
+			Kind: machineDeploymentKind,
+			Name: "foo",
+			UID:  uuid1,
+		},
 	}} {
 		t.Run(tc.description, func(t *testing.T) {
-			owned := machineSetIsOwnedByMachineDeployment(&tc.machineSet, &tc.machineDeployment)
-			if tc.owned != owned {
-				t.Errorf("expected %t, got %t", tc.owned, owned)
+			tc.machineSet.SetOwnerReferences(tc.machineSetOwnerRefs)
+
+			ownerRef := getOwnerForKind(tc.machineSet, machineDeploymentKind)
+			if !reflect.DeepEqual(tc.expectedOwnerRef, ownerRef) {
+				t.Errorf("expected %v, got %v", tc.expectedOwnerRef, ownerRef)
 			}
 		})
 	}
 }
 
-func TestUtilMachineIsOwnedByMachineSet(t *testing.T) {
+func TestUtilGetOwnerByKindMachine(t *testing.T) {
 	for _, tc := range []struct {
-		description string
-		machine     Machine
-		machineSet  MachineSet
-		owned       bool
+		description      string
+		machine          *unstructured.Unstructured
+		machineOwnerRefs []metav1.OwnerReference
+		expectedOwnerRef *metav1.OwnerReference
 	}{{
-		description: "not owned as no owner references",
-		machine:     Machine{},
-		machineSet:  MachineSet{},
-		owned:       false,
+		description:      "not owned as no owner references",
+		machine:          &unstructured.Unstructured{},
+		machineOwnerRefs: []metav1.OwnerReference{},
+		expectedOwnerRef: nil,
 	}, {
 		description: "not owned as not the same Kind",
-		machine: Machine{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "Other",
-				}},
+		machine: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
 			},
 		},
-		machineSet: MachineSet{},
-		owned:      false,
-	}, {
-		description: "not owned because no OwnerReference.Name",
-		machine: Machine{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineSet",
-					UID:  uuid1,
-				}},
-			},
-		},
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				UID: uuid1,
-			},
-		},
-		owned: false,
-	}, {
-		description: "not owned as UID values don't match",
-		machine: Machine{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineSet",
-					Name: "foo",
-					UID:  uuid2,
-				}},
-			},
-		},
-		machineSet: MachineSet{
-			TypeMeta: v1.TypeMeta{
-				Kind: "MachineSet",
-			},
-			ObjectMeta: v1.ObjectMeta{
-				UID: uuid1,
-			},
-		},
-		owned: false,
-	}, {
-		description: "owned as UID values match and same Kind and Name not empty",
-		machine: Machine{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineSet",
-					Name: "foo",
-					UID:  uuid1,
-				}},
-			},
-		},
-		machineSet: MachineSet{
-			TypeMeta: v1.TypeMeta{
-				Kind: "MachineSet",
-			},
-			ObjectMeta: v1.ObjectMeta{
+		machineOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: "Other",
 				Name: "foo",
 				UID:  uuid1,
 			},
 		},
-		owned: true,
+		expectedOwnerRef: nil,
+	}, {
+		description: "not owned because no OwnerReference.Name",
+		machine: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
+			},
+		},
+		machineOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: machineSetKind,
+				UID:  uuid1,
+			},
+		},
+		expectedOwnerRef: nil,
+	}, {
+		description: "owned as UID values match and same Kind and Name not empty",
+		machine: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
+			},
+		},
+		machineOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: machineSetKind,
+				Name: "foo",
+				UID:  uuid2,
+			},
+		},
+		expectedOwnerRef: &metav1.OwnerReference{
+			Kind: machineSetKind,
+			Name: "foo",
+			UID:  uuid2,
+		},
 	}} {
 		t.Run(tc.description, func(t *testing.T) {
-			owned := machineIsOwnedByMachineSet(&tc.machine, &tc.machineSet)
-			if tc.owned != owned {
-				t.Errorf("expected %t, got %t", tc.owned, owned)
+			tc.machine.SetOwnerReferences(tc.machineOwnerRefs)
+
+			ownerRef := getOwnerForKind(tc.machine, machineSetKind)
+			if !reflect.DeepEqual(tc.expectedOwnerRef, ownerRef) {
+				t.Errorf("expected %v, got %v", tc.expectedOwnerRef, ownerRef)
 			}
 		})
 	}
 }
 
-func TestUtilMachineSetMachineDeploymentOwnerRef(t *testing.T) {
+func TestUtilMachineSetHasMachineDeploymentOwnerRef(t *testing.T) {
 	for _, tc := range []struct {
-		description       string
-		machineSet        MachineSet
-		machineDeployment MachineDeployment
-		owned             bool
+		description         string
+		machineSet          *unstructured.Unstructured
+		machineSetOwnerRefs []metav1.OwnerReference
+		owned               bool
 	}{{
-		description:       "machineset not owned as no owner references",
-		machineSet:        MachineSet{},
-		machineDeployment: MachineDeployment{},
-		owned:             false,
+		description:         "machineset not owned as no owner references",
+		machineSet:          &unstructured.Unstructured{},
+		machineSetOwnerRefs: []metav1.OwnerReference{},
+		owned:               false,
 	}, {
 		description: "machineset not owned as ownerref not a MachineDeployment",
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "Other",
-				}},
+		machineSet: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
 			},
 		},
-		machineDeployment: MachineDeployment{},
-		owned:             false,
+		machineSetOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: "Other",
+			},
+		},
+		owned: false,
 	}, {
 		description: "machineset owned as Kind matches and Name not empty",
-		machineSet: MachineSet{
-			ObjectMeta: v1.ObjectMeta{
-				OwnerReferences: []v1.OwnerReference{{
-					Kind: "MachineDeployment",
-					Name: "foo",
-				}},
+		machineSet: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "test",
+					"namespace": "default",
+				},
+				"spec":   map[string]interface{}{},
+				"status": map[string]interface{}{},
 			},
 		},
-		machineDeployment: MachineDeployment{
-			TypeMeta: v1.TypeMeta{
-				Kind: "MachineDeployment",
-			},
-			ObjectMeta: v1.ObjectMeta{
+		machineSetOwnerRefs: []metav1.OwnerReference{
+			{
+				Kind: machineDeploymentKind,
 				Name: "foo",
 			},
 		},
 		owned: true,
 	}} {
 		t.Run(tc.description, func(t *testing.T) {
-			owned := machineSetHasMachineDeploymentOwnerRef(&tc.machineSet)
+			tc.machineSet.SetOwnerReferences(tc.machineSetOwnerRefs)
+			owned := machineSetHasMachineDeploymentOwnerRef(tc.machineSet)
 			if tc.owned != owned {
 				t.Errorf("expected %t, got %t", tc.owned, owned)
 			}
@@ -616,3 +660,249 @@ func TestParseMaxPodsCapacity(t *testing.T) {
 		})
 	}
 }
+
+func Test_clusterNameFromResource(t *testing.T) {
+	for _, tc := range []struct {
+		name     string
+		resource *unstructured.Unstructured
+		want     string
+	}{{
+		name: "cluster name not set, v1alpha1 MachineSet",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.k8s.io/v1alpha1",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "",
+	}, {
+		name: "cluster name not set, v1alpha1 MachineDeployment",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineDeploymentKind,
+				"apiVersion": "cluster.k8s.io/v1alpha1",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "",
+	}, {
+		name: "cluster name set in MachineSet labels, v1alpha1 MachineSet",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.k8s.io/v1alpha1",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+					"labels": map[string]interface{}{
+						deprecatedClusterNameLabel: "bar",
+					},
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name set in MachineDeployment, v1alpha1 MachineDeployment",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineDeploymentKind,
+				"apiVersion": "cluster.k8s.io/v1alpha1",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+					"labels": map[string]interface{}{
+						deprecatedClusterNameLabel: "bar",
+					},
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name set in Machine template labels, v1alpha1 MachineSet",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.k8s.io/v1alpha1",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+					"template": map[string]interface{}{
+						"metadata": map[string]interface{}{
+							"labels": map[string]interface{}{
+								deprecatedClusterNameLabel: "bar",
+							},
+						},
+					},
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name set in Machine template, v1alpha1 MachineDeployment",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineDeploymentKind,
+				"apiVersion": "cluster.k8s.io/v1alpha1",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+					"template": map[string]interface{}{
+						"metadata": map[string]interface{}{
+							"labels": map[string]interface{}{
+								deprecatedClusterNameLabel: "bar",
+							},
+						},
+					},
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name not set, v1alpha2 MachineSet",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha2",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "",
+	}, {
+		name: "cluster name not set, v1alpha2 MachineDeployment",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineDeploymentKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha2",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "",
+	}, {
+		name: "cluster name set in MachineSet labels, v1alpha2 MachineSet",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha2",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+					"labels": map[string]interface{}{
+						clusterNameLabel: "bar",
+					},
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name set in MachineDeployment, v1alpha2 MachineDeployment",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineDeploymentKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha2",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+					"labels": map[string]interface{}{
+						clusterNameLabel: "bar",
+					},
+				},
+				"spec": map[string]interface{}{
+					"replicas": int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name set in spec, v1alpha3 MachineSet",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineSetKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"clusterName": "bar",
+					"replicas":    int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}, {
+		name: "cluster name set in spec, v1alpha3 MachineDeployment",
+		resource: &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"kind":       machineDeploymentKind,
+				"apiVersion": "cluster.x-k8s.io/v1alpha3",
+				"metadata": map[string]interface{}{
+					"name":      "foo",
+					"namespace": "default",
+				},
+				"spec": map[string]interface{}{
+					"clusterName": "bar",
+					"replicas":    int64(1),
+				},
+				"status": map[string]interface{}{},
+			},
+		},
+		want: "bar",
+	}} {
+		t.Run(tc.name, func(t *testing.T) {
+			if got := clusterNameFromResource(tc.resource); got != tc.want {
+				t.Errorf("clusterNameFromResource() = %v, want %v", got, tc.want)
+			}
+		})
+	}
+}
