@@ -363,187 +363,210 @@ func TestDrain(t *testing.T) {
 	}
 
 	tests := []struct {
-		description       string
-		pods              []*apiv1.Pod
-		pdbs              []*policyv1.PodDisruptionBudget
-		rcs               []*apiv1.ReplicationController
-		replicaSets       []*appsv1.ReplicaSet
-		expectFatal       bool
-		expectPods        []*apiv1.Pod
-		expectBlockingPod *BlockingPod
+		description         string
+		pods                []*apiv1.Pod
+		pdbs                []*policyv1.PodDisruptionBudget
+		rcs                 []*apiv1.ReplicationController
+		replicaSets         []*appsv1.ReplicaSet
+		expectFatal         bool
+		expectPods          []*apiv1.Pod
+		expectDaemonSetPods []*apiv1.Pod
+		expectBlockingPod   *BlockingPod
 	}{
 		{
-			description: "RC-managed pod",
-			pods:        []*apiv1.Pod{rcPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			rcs:         []*apiv1.ReplicationController{&rc},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{rcPod},
+			description:         "RC-managed pod",
+			pods:                []*apiv1.Pod{rcPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{rcPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "DS-managed pod",
-			pods:        []*apiv1.Pod{dsPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{},
+			description:         "DS-managed pod",
+			pods:                []*apiv1.Pod{dsPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{},
+			expectDaemonSetPods: []*apiv1.Pod{dsPod},
 		},
 		{
-			description: "DS-managed pod by a custom Daemonset",
-			pods:        []*apiv1.Pod{cdsPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{},
+			description:         "DS-managed pod by a custom Daemonset",
+			pods:                []*apiv1.Pod{cdsPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{},
+			expectDaemonSetPods: []*apiv1.Pod{cdsPod},
 		},
 		{
-			description: "Job-managed pod",
-			pods:        []*apiv1.Pod{jobPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			rcs:         []*apiv1.ReplicationController{&rc},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{jobPod},
+			description:         "Job-managed pod",
+			pods:                []*apiv1.Pod{jobPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{jobPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "SS-managed pod",
-			pods:        []*apiv1.Pod{ssPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			rcs:         []*apiv1.ReplicationController{&rc},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{ssPod},
+			description:         "SS-managed pod",
+			pods:                []*apiv1.Pod{ssPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{ssPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "RS-managed pod",
-			pods:        []*apiv1.Pod{rsPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			replicaSets: []*appsv1.ReplicaSet{&rs},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{rsPod},
+			description:         "RS-managed pod",
+			pods:                []*apiv1.Pod{rsPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			replicaSets:         []*appsv1.ReplicaSet{&rs},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{rsPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "RS-managed pod that is being deleted",
-			pods:        []*apiv1.Pod{rsPodDeleted},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			replicaSets: []*appsv1.ReplicaSet{&rs},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{},
+			description:         "RS-managed pod that is being deleted",
+			pods:                []*apiv1.Pod{rsPodDeleted},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			replicaSets:         []*appsv1.ReplicaSet{&rs},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description:       "naked pod",
-			pods:              []*apiv1.Pod{nakedPod},
-			pdbs:              []*policyv1.PodDisruptionBudget{},
-			expectFatal:       true,
-			expectPods:        []*apiv1.Pod{},
-			expectBlockingPod: &BlockingPod{Pod: nakedPod, Reason: NotReplicated},
+			description:         "naked pod",
+			pods:                []*apiv1.Pod{nakedPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         true,
+			expectPods:          []*apiv1.Pod{},
+			expectBlockingPod:   &BlockingPod{Pod: nakedPod, Reason: NotReplicated},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description:       "pod with EmptyDir",
-			pods:              []*apiv1.Pod{emptydirPod},
-			pdbs:              []*policyv1.PodDisruptionBudget{},
-			rcs:               []*apiv1.ReplicationController{&rc},
-			expectFatal:       true,
-			expectPods:        []*apiv1.Pod{},
-			expectBlockingPod: &BlockingPod{Pod: emptydirPod, Reason: LocalStorageRequested},
+			description:         "pod with EmptyDir",
+			pods:                []*apiv1.Pod{emptydirPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         true,
+			expectPods:          []*apiv1.Pod{},
+			expectBlockingPod:   &BlockingPod{Pod: emptydirPod, Reason: LocalStorageRequested},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "failed pod",
-			pods:        []*apiv1.Pod{failedPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{failedPod},
+			description:         "failed pod",
+			pods:                []*apiv1.Pod{failedPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{failedPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "evicted pod",
-			pods:        []*apiv1.Pod{evictedPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{evictedPod},
+			description:         "evicted pod",
+			pods:                []*apiv1.Pod{evictedPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{evictedPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "pod in terminal state",
-			pods:        []*apiv1.Pod{terminalPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{terminalPod},
+			description:         "pod in terminal state",
+			pods:                []*apiv1.Pod{terminalPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{terminalPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "pod with PodSafeToEvict annotation",
-			pods:        []*apiv1.Pod{safePod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{safePod},
+			description:         "pod with PodSafeToEvict annotation",
+			pods:                []*apiv1.Pod{safePod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{safePod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "kube-system pod with PodSafeToEvict annotation",
-			pods:        []*apiv1.Pod{kubeSystemSafePod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{kubeSystemSafePod},
+			description:         "kube-system pod with PodSafeToEvict annotation",
+			pods:                []*apiv1.Pod{kubeSystemSafePod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{kubeSystemSafePod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "pod with EmptyDir and PodSafeToEvict annotation",
-			pods:        []*apiv1.Pod{emptydirSafePod},
-			pdbs:        []*policyv1.PodDisruptionBudget{},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{emptydirSafePod},
+			description:         "pod with EmptyDir and PodSafeToEvict annotation",
+			pods:                []*apiv1.Pod{emptydirSafePod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{emptydirSafePod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description:       "RC-managed pod with PodSafeToEvict=false annotation",
-			pods:              []*apiv1.Pod{unsafeRcPod},
-			rcs:               []*apiv1.ReplicationController{&rc},
-			pdbs:              []*policyv1.PodDisruptionBudget{},
-			expectFatal:       true,
-			expectPods:        []*apiv1.Pod{},
-			expectBlockingPod: &BlockingPod{Pod: unsafeRcPod, Reason: NotSafeToEvictAnnotation},
+			description:         "RC-managed pod with PodSafeToEvict=false annotation",
+			pods:                []*apiv1.Pod{unsafeRcPod},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			expectFatal:         true,
+			expectPods:          []*apiv1.Pod{},
+			expectBlockingPod:   &BlockingPod{Pod: unsafeRcPod, Reason: NotSafeToEvictAnnotation},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description:       "Job-managed pod with PodSafeToEvict=false annotation",
-			pods:              []*apiv1.Pod{unsafeJobPod},
-			pdbs:              []*policyv1.PodDisruptionBudget{},
-			rcs:               []*apiv1.ReplicationController{&rc},
-			expectFatal:       true,
-			expectPods:        []*apiv1.Pod{},
-			expectBlockingPod: &BlockingPod{Pod: unsafeJobPod, Reason: NotSafeToEvictAnnotation},
+			description:         "Job-managed pod with PodSafeToEvict=false annotation",
+			pods:                []*apiv1.Pod{unsafeJobPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         true,
+			expectPods:          []*apiv1.Pod{},
+			expectBlockingPod:   &BlockingPod{Pod: unsafeJobPod, Reason: NotSafeToEvictAnnotation},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "empty PDB with RC-managed pod",
-			pods:        []*apiv1.Pod{rcPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{emptyPDB},
-			rcs:         []*apiv1.ReplicationController{&rc},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{rcPod},
+			description:         "empty PDB with RC-managed pod",
+			pods:                []*apiv1.Pod{rcPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{emptyPDB},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{rcPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "kube-system PDB with matching kube-system pod",
-			pods:        []*apiv1.Pod{kubeSystemRcPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{kubeSystemPDB},
-			rcs:         []*apiv1.ReplicationController{&kubeSystemRc},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{kubeSystemRcPod},
+			description:         "kube-system PDB with matching kube-system pod",
+			pods:                []*apiv1.Pod{kubeSystemRcPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{kubeSystemPDB},
+			rcs:                 []*apiv1.ReplicationController{&kubeSystemRc},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{kubeSystemRcPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description:       "kube-system PDB with non-matching kube-system pod",
-			pods:              []*apiv1.Pod{kubeSystemRcPod},
-			pdbs:              []*policyv1.PodDisruptionBudget{kubeSystemFakePDB},
-			rcs:               []*apiv1.ReplicationController{&kubeSystemRc},
-			expectFatal:       true,
-			expectPods:        []*apiv1.Pod{},
-			expectBlockingPod: &BlockingPod{Pod: kubeSystemRcPod, Reason: UnmovableKubeSystemPod},
+			description:         "kube-system PDB with non-matching kube-system pod",
+			pods:                []*apiv1.Pod{kubeSystemRcPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{kubeSystemFakePDB},
+			rcs:                 []*apiv1.ReplicationController{&kubeSystemRc},
+			expectFatal:         true,
+			expectPods:          []*apiv1.Pod{},
+			expectBlockingPod:   &BlockingPod{Pod: kubeSystemRcPod, Reason: UnmovableKubeSystemPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description: "kube-system PDB with default namespace pod",
-			pods:        []*apiv1.Pod{rcPod},
-			pdbs:        []*policyv1.PodDisruptionBudget{kubeSystemPDB},
-			rcs:         []*apiv1.ReplicationController{&rc},
-			expectFatal: false,
-			expectPods:  []*apiv1.Pod{rcPod},
+			description:         "kube-system PDB with default namespace pod",
+			pods:                []*apiv1.Pod{rcPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{kubeSystemPDB},
+			rcs:                 []*apiv1.ReplicationController{&rc},
+			expectFatal:         false,
+			expectPods:          []*apiv1.Pod{rcPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 		{
-			description:       "default namespace PDB with matching labels kube-system pod",
-			pods:              []*apiv1.Pod{kubeSystemRcPod},
-			pdbs:              []*policyv1.PodDisruptionBudget{defaultNamespacePDB},
-			rcs:               []*apiv1.ReplicationController{&kubeSystemRc},
-			expectFatal:       true,
-			expectPods:        []*apiv1.Pod{},
-			expectBlockingPod: &BlockingPod{Pod: kubeSystemRcPod, Reason: UnmovableKubeSystemPod},
+			description:         "default namespace PDB with matching labels kube-system pod",
+			pods:                []*apiv1.Pod{kubeSystemRcPod},
+			pdbs:                []*policyv1.PodDisruptionBudget{defaultNamespacePDB},
+			rcs:                 []*apiv1.ReplicationController{&kubeSystemRc},
+			expectFatal:         true,
+			expectPods:          []*apiv1.Pod{},
+			expectBlockingPod:   &BlockingPod{Pod: kubeSystemRcPod, Reason: UnmovableKubeSystemPod},
+			expectDaemonSetPods: []*apiv1.Pod{},
 		},
 	}
 
@@ -569,7 +592,7 @@ func TestDrain(t *testing.T) {
 
 		registry := kube_util.NewListerRegistry(nil, nil, nil, nil, nil, dsLister, rcLister, jobLister, rsLister, ssLister)
 
-		pods, blockingPod, err := GetPodsForDeletionOnNodeDrain(test.pods, test.pdbs, true, true, true, registry, 0, time.Now())
+		pods, daemonSetPods, blockingPod, err := GetPodsForDeletionOnNodeDrain(test.pods, test.pdbs, true, true, true, registry, 0, time.Now())
 
 		if test.expectFatal {
 			assert.Equal(t, test.expectBlockingPod, blockingPod)
@@ -588,5 +611,7 @@ func TestDrain(t *testing.T) {
 		if len(pods) != len(test.expectPods) {
 			t.Fatalf("Wrong pod list content: %v", test.description)
 		}
+
+		assert.ElementsMatch(t, test.expectDaemonSetPods, daemonSetPods)
 	}
 }
